'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.noSuchFile = exports.fileExist = undefined;

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

var _mkdirp = require('mkdirp');

var _mkdirp2 = _interopRequireDefault(_mkdirp);

var _httpErrors = require('http-errors');

var _httpErrors2 = _interopRequireDefault(_httpErrors);

var _streams = require('@verdaccio/streams');

var _fileLocking = require('@verdaccio/file-locking');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var fileExist = exports.fileExist = 'EEXISTS';
var noSuchFile = exports.noSuchFile = 'ENOENT';

var resourceNotAvailable = 'EAGAIN';
var pkgFileName = 'package.json';
var fSError = function fSError(message) {
  var code = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 409;

  var err = (0, _httpErrors2.default)(code, message);
  // $FlowFixMe
  err.code = message;

  return err;
};

var ErrorCode = {
  get503: function get503() {
    return fSError('resource temporarily unavailable', 500);
  },
  get404: function get404(customMessage) {
    return fSError('no such package available', 404);
  }
};

var tempFile = function tempFile(str) {
  return `${str}.tmp${String(Math.random()).substr(2)}`;
};

var renameTmp = function renameTmp(src, dst, _cb) {
  var cb = function cb(err) {
    if (err) {
      _fs2.default.unlink(src, function () {});
    }
    _cb(err);
  };

  if (process.platform !== 'win32') {
    return _fs2.default.rename(src, dst, cb);
  }

  // windows can't remove opened file,
  // but it seem to be able to rename it
  var tmp = tempFile(dst);
  _fs2.default.rename(dst, tmp, function (err) {
    _fs2.default.rename(src, dst, cb);
    if (!err) {
      _fs2.default.unlink(tmp, function () {});
    }
  });
};

var LocalFS = function () {
  function LocalFS(path, logger) {
    _classCallCheck(this, LocalFS);

    this.path = path;
    this.logger = logger;
  }

  /**
   *  This function allows to update the package thread-safely
     Algorithm:
     1. lock package.json for writing
     2. read package.json
     3. updateFn(pkg, cb), and wait for cb
     4. write package.json.tmp
     5. move package.json.tmp package.json
     6. callback(err?)
   * @param {*} name
   * @param {*} updateHandler
   * @param {*} onWrite
   * @param {*} transformPackage
   * @param {*} onEnd
   */


  _createClass(LocalFS, [{
    key: 'updatePackage',
    value: function updatePackage(name, updateHandler, onWrite, transformPackage, onEnd) {
      var _this = this;

      this._lockAndReadJSON(pkgFileName, function (err, json) {
        var locked = false;
        var self = _this;
        // callback that cleans up lock first
        var unLockCallback = function unLockCallback(lockError) {
          var _args = arguments;
          if (locked) {
            self._unlockJSON(pkgFileName, function () {
              // ignore any error from the unlock
              onEnd.apply(lockError, _args);
            });
          } else {
            onEnd.apply(undefined, _toConsumableArray(_args));
          }
        };

        if (!err) {
          locked = true;
        }

        if (_lodash2.default.isNil(err) === false) {
          if (err.code === resourceNotAvailable) {
            return unLockCallback(ErrorCode.get503());
          } else if (err.code === noSuchFile) {
            return unLockCallback(ErrorCode.get404());
          } else {
            return unLockCallback(err);
          }
        }

        updateHandler(json, function (err) {
          if (err) {
            return unLockCallback(err);
          }
          onWrite(name, transformPackage(json), unLockCallback);
        });
      });
    }
  }, {
    key: 'deletePackage',
    value: function deletePackage(fileName, callback) {
      return _fs2.default.unlink(this._getStorage(fileName), callback);
    }
  }, {
    key: 'removePackage',
    value: function removePackage(callback) {
      _fs2.default.rmdir(this._getStorage('.'), callback);
    }
  }, {
    key: 'createPackage',
    value: function createPackage(name, value, cb) {
      this._createFile(this._getStorage(name), this._convertToString(value), cb);
    }
  }, {
    key: 'savePackage',
    value: function savePackage(name, value, cb) {
      this._writeFile(this._getStorage(name), this._convertToString(value), cb);
    }
  }, {
    key: 'readPackage',
    value: function readPackage(name, cb) {
      this._readStorageFile(this._getStorage(name)).then(function (res) {
        try {
          var data = JSON.parse(res.toString('utf8'));

          cb(null, data);
        } catch (err) {
          cb(err);
        }
      }, function (err) {
        return cb(err);
      });
    }
  }, {
    key: 'writeTarball',
    value: function writeTarball(name) {
      var uploadStream = new _streams.UploadTarball();

      var _ended = 0;
      uploadStream.on('end', function () {
        _ended = 1;
      });

      var pathName = this._getStorage(name);

      _fs2.default.exists(pathName, function (exists) {
        if (exists) {
          return uploadStream.emit('error', fSError(fileExist));
        }

        var temporalName = `${name}.tmp-${String(Math.random()).replace(/^0\./, '')}`;
        var file = _fs2.default.createWriteStream(temporalName);
        var opened = false;
        uploadStream.pipe(file);

        uploadStream.done = function () {
          var onend = function onend() {
            file.on('close', function () {
              renameTmp(temporalName, pathName, function (err) {
                if (err) {
                  uploadStream.emit('error', err);
                } else {
                  uploadStream.emit('success');
                }
              });
            });
            file.end();
          };
          if (_ended) {
            onend();
          } else {
            uploadStream.on('end', onend);
          }
        };
        uploadStream.abort = function () {
          if (opened) {
            opened = false;
            file.on('close', function () {
              _fs2.default.unlink(temporalName, function () {});
            });
          }
          file.end();
        };
        file.on('open', function () {
          opened = true;
          // re-emitting open because it's handled in storage.js
          uploadStream.emit('open');
        });
        file.on('error', function (err) {
          uploadStream.emit('error', err);
        });
      });
      return uploadStream;
    }
  }, {
    key: 'readTarball',
    value: function readTarball(name, readTarballStream) {
      var callback = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : function () {};

      var pathName = this._getStorage(name);

      var readStream = _fs2.default.createReadStream(pathName);

      readStream.on('error', function (err) {
        readTarballStream.emit('error', err);
      });

      readStream.on('open', function (fd) {
        _fs2.default.fstat(fd, function (err, stats) {
          if (_lodash2.default.isNil(err) === false) {
            return readTarballStream.emit('error', err);
          }
          readTarballStream.emit('content-length', stats.size);
          readTarballStream.emit('open');
          readStream.pipe(readTarballStream);
        });
      });

      readTarballStream = new _streams.ReadTarball();

      readTarballStream.abort = function () {
        readStream.close();
      };

      return readTarballStream;
    }
  }, {
    key: '_createFile',
    value: function _createFile(name, contents, callback) {
      var _this2 = this;

      _fs2.default.exists(name, function (exists) {
        if (exists) {
          return callback(fSError(fileExist));
        }
        _this2._writeFile(name, contents, callback);
      });
    }
  }, {
    key: '_readStorageFile',
    value: function _readStorageFile(name) {
      return new Promise(function (resolve, reject) {
        _fs2.default.readFile(name, function (err, data) {
          if (err) {
            reject(err);
          } else {
            resolve(data);
          }
        });
      });
    }
  }, {
    key: '_convertToString',
    value: function _convertToString(value) {
      return JSON.stringify(value, null, '\t');
    }
  }, {
    key: '_getStorage',
    value: function _getStorage() {
      var name = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : '';

      var storagePath = _path2.default.join(this.path, name);

      return storagePath;
    }
  }, {
    key: '_writeFile',
    value: function _writeFile(dest, data, cb) {
      var createTempFile = function createTempFile(cb) {
        var tempFilePath = tempFile(dest);

        _fs2.default.writeFile(tempFilePath, data, function (err) {
          if (err) {
            return cb(err);
          }
          renameTmp(tempFilePath, dest, cb);
        });
      };

      createTempFile(function (err) {
        if (err && err.code === noSuchFile) {
          (0, _mkdirp2.default)(_path2.default.dirname(dest), function (err) {
            if (err) {
              return cb(err);
            }
            createTempFile(cb);
          });
        } else {
          cb(err);
        }
      });
    }
  }, {
    key: '_lockAndReadJSON',
    value: function _lockAndReadJSON(name, cb) {
      var fileName = this._getStorage(name);

      (0, _fileLocking.readFile)(fileName, {
        lock: true,
        parse: true
      }, function (err, res) {
        if (err) {
          return cb(err);
        }
        return cb(null, res);
      });
    }
  }, {
    key: '_unlockJSON',
    value: function _unlockJSON(name, cb) {
      (0, _fileLocking.unlockFile)(this._getStorage(name), cb);
    }
  }]);

  return LocalFS;
}();

exports.default = LocalFS;